#!python3.9

import json
from . import Message
from xml.etree.ElementTree import Element

class VoiceMessage(Message):

    # message type 
    type:str = 'voice'

    def __init__(self, to_username: str, from_username: str, agent_id: str, media_id: str, format:str = 'amr', **kwargs) -> None:
        super().__init__(to_username, from_username, agent_id, **kwargs)
        # media id for the voice 
        self.media_id:str = media_id
        # the format of the voice file 
        self.format:str = format

    def to_xml(self) -> str:
        '''Represent voice message in XML format'''
        return f'''<xml><ToUserName><![CDATA[{self.to_username}]]></ToUserName><FromUserName><![CDATA[{self.from_username}]]></FromUserName><CreateTime>{self.create_time}</CreateTime><MsgType><![CDATA[voice]]></MsgType><Voice><MediaId><![CDATA[{self.media_id}]]></MediaId></Voice><MsgId>{self.message_id}</MsgId><AgentID>{self.agent_id}</AgentID></xml>'''

    def to_json(self) -> str:
        '''Represent voice message in JSON format'''
        return json.dumps({"touser":self.to_username,"msgtype":"voice","agentid":self.agent_id,"voice":{"media_id":self.media_id},"safe":1 if self.safe else 0,"enable_id_trans":1 if self.enable_id_trans else 0,"enable_duplicate_check":1 if self.enable_duplicate_check else 0,"duplicate_check_interval":self.duplicate_check_interval})
    
    @classmethod
    def from_xml(cls, xml_tree: Element):
        '''Parse voice message from XML object'''
        to_username:str = xml_tree.find('ToUserName').text
        from_username:str = xml_tree.find('FromUserName').text
        create_time:str = xml_tree.find('CreateTime').text
        media_id:str = xml_tree.find('MediaId').text
        format:str = xml_tree.find('Format').text
        msg_id:str = xml_tree.find('MsgId').text
        agent_id:str = xml_tree.find('AgentID').text

        return cls(to_username, from_username, media_id, format=format, agent_id=agent_id, create_time=create_time, msg_id=msg_id)




#!python3.9

import json
from . import Message
from xml.etree.ElementTree import Element

class TextMessage(Message):

    # message type 
    type:str = 'text'

    def __init__(self, to_username: str, from_username: str, agent_id: str, content: str, **kwargs) -> None:
        super().__init__(to_username, from_username, agent_id, **kwargs)
        # the content of the message 
        self.content:str = content

    def to_xml(self) -> str:
        '''Represent the text message in XML format'''
        return f'''<xml><ToUserName><![CDATA[{self.to_username}]]></ToUserName><FromUserName><![CDATA[{self.from_username}]]></FromUserName><CreateTime>{self.create_time}</CreateTime><MsgType><![CDATA[text]]></MsgType><Content><![CDATA[{self.content}]]></Content><MsgId>{self.message_id}</MsgId><AgentID>{self.agent_id}</AgentID></xml>'''
        
    def to_json(self) -> str:
        '''Represent the text message in JSON format'''
        return json.dumps({"touser":self.to_username,"msgtype":"text","agentid":self.agent_id,"text":{"content":self.content},"safe":1 if self.safe else 0,"enable_id_trans":1 if self.enable_id_trans else 0,"enable_duplicate_check":1 if self.enable_duplicate_check else 0,"duplicate_check_interval":self.duplicate_check_interval})
    
    @classmethod
    def from_xml(cls, xml_tree: Element):
        '''Parse text message from XML object'''
        to_username:str = xml_tree.find('ToUserName').text
        from_username:str = xml_tree.find('FromUserName').text
        create_time:str = xml_tree.find('CreateTime').text
        content:str = xml_tree.find('Content').text
        msg_id:str = xml_tree.find('MsgId').text
        agent_id:str = xml_tree.find('AgentID').text

        return cls(to_username, from_username, content, agent_id=agent_id, create_time=create_time, msg_id=msg_id)




#!python3.9

import json
from . import Message
from xml.etree.ElementTree import Element

class FileMessage(Message):

    # message type 
    type:str = 'file'

    def __init__(self, to_username: str, from_username: str, agent_id: str, media_id: str, **kwargs) -> None:
        super().__init__(to_username, from_username, agent_id, **kwargs)
        # media id for the file 
        self.media_id:str = media_id

    def to_xml(self) -> str:
        '''Represent file message in XML format'''
        raise NotImplementedError('Unable to reply with a file message in current WeWork')

    def to_json(self) -> str:
        '''Represent file message in JSON format'''
        return json.dumps({"touser":self.to_username,"msgtype":"file","agentid":self.agent_id,"file":{"media_id":self.media_id},"safe":1 if self.safe else 0,"enable_id_trans":1 if self.enable_id_trans else 0,"enable_duplicate_check":1 if self.enable_duplicate_check else 0,"duplicate_check_interval":self.duplicate_check_interval})
    
    @classmethod
    def from_xml(cls, xml_tree: Element):
        '''Parse file message from XML object'''
        raise NotImplementedError('Unable to receive and parse a file message in current WeWork')

